@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002-2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002-2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002-2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002-2004 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -26,6 +26,7 @@
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input-stdio.h>
 #include <stdio.h>
+#include <string.h>
 #include "libwpd.h"
 #include "GSFStream.h"
 #include "TextListenerImpl.h"
@@ -34,18 +35,34 @@ int main(int argc, char *argv[])
 {
 	if (argc < 2)
 	{
-		printf("Usage: wpd2text <WordPerfect Document>\n");
+		printf("Usage: wpd2text [--info] <WordPerfect Document>\n");
+		printf("Use \"--info\" to get document metadata instead\n");
+		printf("of the document itself\n");
 		return -1;
 	}
 	gsf_init();
 
-	GError   *err;
-	GsfInput * input = GSF_INPUT(gsf_input_stdio_new (argv[1], &err));
+	GError   *err = NULL;
+        char *szInputFile;
+	bool isInfo;
+
+        if (!strcmp(argv[1], "--info"))
+	{
+                isInfo = true;
+		szInputFile = argv[2];
+	}		
+	else
+	{
+		isInfo = false;
+                szInputFile = argv[1];
+	}
+
+	GsfInput * input = GSF_INPUT(gsf_input_stdio_new (szInputFile, &err));
 	if (input == NULL) 
 	{
 		g_return_val_if_fail (err != NULL, 1);
 		
-		g_warning ("'%s' error: %s", argv[1], err->message);
+		g_warning ("'%s' error: %s", szInputFile, err->message);
 		g_error_free (err);
 		return 1;
 	}
@@ -56,10 +73,11 @@ int main(int argc, char *argv[])
 	if (confidence == WPD_CONFIDENCE_NONE || confidence == WPD_CONFIDENCE_POOR)
 	{
 		printf("ERROR: Unsupported file format!\n");
+		delete gsfInput;
 		return 1;
 	}
 	
-	TextListenerImpl listenerImpl;
+	TextListenerImpl listenerImpl(isInfo);
  	WPDResult error = WPDocument::parse(gsfInput, static_cast<WPXHLListenerImpl *>(&listenerImpl));
 
 	if (error == WPD_FILE_ACCESS_ERROR)
