@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2004 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ class WP3EndOfLinePageGroup : public WP3VariableLengthGroup
 {
 public:
 	WP3EndOfLinePageGroup(WPXInputStream *input);	
-	virtual void parse(WP3Listener *listener);
+	void parse(WP3Listener *listener);
 	
 protected:
-	virtual void _readContents(WPXInputStream *input);
+	void _readContents(WPXInputStream *input);
 };
 
 #endif /* WP3ENDOFLINEPAGEGROUP_H */
