@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2004 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2004 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
 #include "WP3Parser.h"
 #include "WPXHeader.h"
 #include "WP3Part.h"
-#include "WP3Listener.h"
+#include "WP3ContentListener.h"
 #include "WP3StylesListener.h"
 #include "libwpd_internal.h"
 #include "WPXTable.h"
@@ -89,34 +89,53 @@ void WP3Parser::parseDocument(WPXInputStream *input, WP3Listener *listener)
 void WP3Parser::parse(WPXHLListenerImpl *listenerImpl)
 {
 	WPXInputStream *input = getInput();
-	std::vector<WPXPageSpan *> pageList;
-	WPXTableList tableList;	
+	std::list<WPXPageSpan> pageList;
+	WPXTableList tableList;
+	std::vector<WP3SubDocument *> subDocuments;	
 	
 	try
  	{
 		// do a "first-pass" parse of the document
 		// gather table border information, page properties (per-page)
-		WP3StylesListener stylesListener(&pageList, tableList);
+		WP3StylesListener stylesListener(pageList, tableList, subDocuments);
 		parse(input, &stylesListener);
 
+		// postprocess the pageList == remove duplicate page spans due to the page breaks
+		std::list<WPXPageSpan>::iterator previousPage = pageList.begin();
+		for (std::list<WPXPageSpan>::iterator Iter=pageList.begin(); Iter != pageList.end(); /* Iter++ */)
+		{
+			if ((Iter != previousPage) && (*previousPage==*Iter))
+			{
+				(*previousPage).setPageSpan((*previousPage).getPageSpan() + (*Iter).getPageSpan());
+				Iter = pageList.erase(Iter);
+			}
+			else
+			{
+				previousPage = Iter;
+				Iter++;
+			}
+		}
+
 		// second pass: here is where we actually send the messages to the target app
 		// that are necessary to emit the body of the target document
-		WP3Listener listener(&pageList, listenerImpl); // FIXME: SHOULD BE CONTENT_LISTENER, AND SHOULD BE PASSED TABLE DATA!
+		WP3ContentListener listener(pageList, subDocuments, listenerImpl); // FIXME: SHOULD BE CONTENT_LISTENER, AND SHOULD BE PASSED TABLE DATA!
 		parse(input, &listener);
 		
 		// cleanup section: free the used resources
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
+		for (std::vector<WP3SubDocument *>::iterator iterSubDoc = subDocuments.begin(); iterSubDoc != subDocuments.end(); iterSubDoc++)
 		{
-			delete *iterSpan;
+			if (*iterSubDoc)
+				delete *iterSubDoc;
 		}
 	}
 	catch(FileException)
 	{
 		WPD_DEBUG_MSG(("WordPerfect: File Exception. Parse terminated prematurely."));
 
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
+		for (std::vector<WP3SubDocument *>::iterator iterSubDoc = subDocuments.begin(); iterSubDoc != subDocuments.end(); iterSubDoc++)
 		{
-			delete *iterSpan;
+			if (*iterSubDoc)
+				delete *iterSubDoc;
 		}
 
 		throw FileException();
