@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
- * Copyright (C) 2005 Fridrich Strba (fridrich.strba@bluewin.ch)
+ * Copyright (C) 2004 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2005-2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,9 @@ class WP3VariableLengthGroup : public WP3Part
 	WP3VariableLengthGroup(); // WP3VariableLengthGroup should _never_ be constructed, only its inherited classes
 	virtual ~WP3VariableLengthGroup() {}
 	
-	static WP3VariableLengthGroup * constructVariableLengthGroup(WPXInputStream *input, uint8_t group);
+	static WP3VariableLengthGroup * constructVariableLengthGroup(WPXInputStream *input, const uint8_t group);
+
+	static bool isGroupConsistent(WPXInputStream *input, const uint8_t group);
 
  protected:
 	void _read(WPXInputStream *input);
