@@ -1,6 +1,7 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +15,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -37,8 +38,9 @@ WPDConfidence WP42Heuristics::isWP42FileFormat(WPXInputStream *input, bool parti
 	
 	while (!input->atEOS())
 	{
-		uint8_t readVal;
-		readVal = readU8(input);
+		uint8_t readVal = readU8(input);
+
+		WPD_DEBUG_MSG(("WP42Heuristics, Offset 0x%.8x, value 0x%.2x\n", input->tell() - 1, readVal));
 		
 		if (readVal < (uint8_t)0x20)
 		{
@@ -51,6 +53,12 @@ WPDConfidence WP42Heuristics::isWP42FileFormat(WPXInputStream *input, bool parti
 		else if (readVal >= (uint8_t)0x80 && readVal <= (uint8_t)0xBF)
 		{
 			// single character function codes, skip
+			functionGroupCount++;
+		}
+		else if (readVal >= (uint8_t)0xFF)
+		{
+			// special codes that should not be found as separate functions
+			return WPD_CONFIDENCE_NONE;
 		}
 		else 
 		{
@@ -82,7 +90,7 @@ WPDConfidence WP42Heuristics::isWP42FileFormat(WPXInputStream *input, bool parti
 				// fixed length function group
 				
 				// seek to the position where the closing gate should be
-				bool res = input->seek(WP42_FUNCTION_GROUP_SIZE[readVal-0xC0]-2, WPX_SEEK_CUR);
+				int res = input->seek(WP42_FUNCTION_GROUP_SIZE[readVal-0xC0]-2, WPX_SEEK_CUR);
 				// when passed the complete file, we should be able to do that
 				if (!partialContent && res)
 					return WPD_CONFIDENCE_NONE;
