@@ -1,6 +1,7 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +15,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -29,7 +30,8 @@
 #include "libwpd_internal.h"
 #include "WPXTable.h"
 #include "WP42FileStructure.h"
-#include "WP42HLStylesListener.h"
+#include "WP42StylesListener.h"
+#include "WP42ContentListener.h"
 
 WP42Parser::WP42Parser(WPXInputStream *input) :
 	WPXParser(input, NULL)
@@ -40,7 +42,7 @@ WP42Parser::~WP42Parser()
 {
 }
 
-void WP42Parser::parse(WPXInputStream *input, WP42HLListener *listener)
+void WP42Parser::parse(WPXInputStream *input, WP42Listener *listener)
 {
 	listener->startDocument();
 
@@ -54,7 +56,7 @@ void WP42Parser::parse(WPXInputStream *input, WP42HLListener *listener)
 }
 
 // parseDocument: parses a document body (may call itself recursively, on other streams, or itself)
-void WP42Parser::parseDocument(WPXInputStream *input, WP42HLListener *listener)
+void WP42Parser::parseDocument(WPXInputStream *input, WP42Listener *listener)
 {
 	while (!input->atEOS())
 	{
@@ -63,6 +65,8 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42HLListener *listener)
 
 		if (readVal < (uint8_t)0x20)
 		{
+			WPD_DEBUG_MSG(("Offset: %i, Handling Control Character 0x%2x\n", input->tell(), readVal));			
+
 			switch (readVal)
 			{
 				case 0x09: // tab
@@ -72,13 +76,13 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42HLListener *listener)
 					listener->insertEOL();
 					break;
 				case 0x0B: // soft new page
-					listener->insertBreak(WPX_PAGE_BREAK);
+					listener->insertBreak(WPX_SOFT_PAGE_BREAK);
 					break;
 				case 0x0C: // hard new page
 					listener->insertBreak(WPX_PAGE_BREAK);
 					break;
 				case 0x0D: // soft new line
-					listener->insertEOL();
+					listener->insertCharacter(' ');
 					break;
 				default:
 					// unsupported or undocumented token, ignore
@@ -87,11 +91,15 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42HLListener *listener)
 		}
 		else if (readVal >= (uint8_t)0x20 && readVal <= (uint8_t)0x7F)
 		{
+			WPD_DEBUG_MSG(("Offset: %i, Handling Ascii Character 0x%2x\n", input->tell(), readVal));			
+
 			// normal ASCII characters
 			listener->insertCharacter( readVal );
 		}
 		else if (readVal >= (uint8_t)0x80 && readVal <= (uint8_t)0xBF)
 		{
+			WPD_DEBUG_MSG(("Offset: %i, Handling Single Character Function 0x%2x\n", input->tell(), readVal));			
+
 			// single character function codes
 			switch (readVal)
 			{
@@ -140,49 +148,69 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42HLListener *listener)
 					break;
 			}
 		}
-		else
+		else if (readVal >= (uint8_t)0xC0 && readVal <= (uint8_t)0xFE)
 		{
 			WP42Part *part = WP42Part::constructPart(input, readVal);
 			if (part != NULL)
 			{
-				//part->parse(llListener);
-				delete(part);
+				part->parse(listener);
+				DELETEP(part);
 			}
 		}
+		// ignore the rest since they are not documented and at least 0xFF is a special character that
+		// marks end of variable length part in variable length multi-byte functions
 	}
 }
 
 void WP42Parser::parse(WPXHLListenerImpl *listenerImpl)
 {
 	WPXInputStream *input = getInput();
-	std::vector<WPXPageSpan *> pageList;
-	WPXTableList tableList;
-
+	std::list<WPXPageSpan> pageList;
+	std::vector<WP42SubDocument *> subDocuments;	
+	
 	try
  	{
 		// do a "first-pass" parse of the document
 		// gather table border information, page properties (per-page)
-		WP42HLStylesListener stylesListener(&pageList, tableList);
+		WP42StylesListener stylesListener(pageList, subDocuments);
 		parse(input, &stylesListener);
 
+		// postprocess the pageList == remove duplicate page spans due to the page breaks
+		std::list<WPXPageSpan>::iterator previousPage = pageList.begin();
+		for (std::list<WPXPageSpan>::iterator Iter=pageList.begin(); Iter != pageList.end();)
+		{
+			if ((Iter != previousPage) && ((*previousPage)==(*Iter)))
+			{
+				(*previousPage).setPageSpan((*previousPage).getPageSpan() + (*Iter).getPageSpan());
+				Iter = pageList.erase(Iter);
+			}
+			else
+			{
+				previousPage = Iter;
+				Iter++;
+			}
+		}
+
 		// second pass: here is where we actually send the messages to the target app
 		// that are necessary to emit the body of the target document
-		WP42HLListener listener(&pageList, listenerImpl); // FIXME: SHOULD BE CONTENT_LISTENER, AND SHOULD BE PASSED TABLE DATA!
+		WP42ContentListener listener(pageList, subDocuments, listenerImpl);
 		parse(input, &listener);
 
 		// cleanup section: free the used resources
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
+		for (std::vector<WP42SubDocument *>::iterator iterSubDoc = subDocuments.begin(); iterSubDoc != subDocuments.end(); iterSubDoc++)
 		{
-			delete *iterSpan;
+			if (*iterSubDoc)
+				delete (*iterSubDoc);
 		}
 	}
 	catch(FileException)
 	{
 		WPD_DEBUG_MSG(("WordPerfect: File Exception. Parse terminated prematurely."));
 
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
+		for (std::vector<WP42SubDocument *>::iterator iterSubDoc = subDocuments.begin(); iterSubDoc != subDocuments.end(); iterSubDoc++)
 		{
-			delete *iterSpan;
+			if (*iterSubDoc)
+				delete (*iterSubDoc);
 		}
 
 		throw FileException();
