@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,7 +37,7 @@ public:
 	WP42Parser(WPXInputStream *input);
 	~WP42Parser();
 
-	virtual void parse(WPXHLListenerImpl *listenerImpl);
+	void parse(WPXHLListenerImpl *listenerImpl);
 
 	static void parseDocument(WPXInputStream *input, WP42Listener *listener);
 
