@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,10 +31,10 @@ class WP6DefaultInitialFontPacket : public WP6PrefixDataPacket
 {
  public:
 	WP6DefaultInitialFontPacket(WPXInputStream *input, int id, uint32_t dataOffset, uint32_t dataSize);
-	virtual void _readContents(WPXInputStream *input);
+	void _readContents(WPXInputStream *input);
 	const uint16_t getInitialFontDescriptorPID() const { return m_initialFontDescriptorPID; }
 	const uint16_t getPointSize() const { return m_pointSize; }
-	virtual void parse(WP6Listener *listener) const;
+	void parse(WP6Listener *listener) const;
 	
  private:
 	uint16_t m_numPrefixIDs;
