@@ -1,7 +1,7 @@
 /* libwpd
  * Copyright (C) 2003 Net Integration Technologies
- * Copyright (C) 2002-2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002-2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2004 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
@@ -57,7 +57,6 @@ WP6EOLGroup::WP6EOLGroup(WPXInputStream *input) :
 	m_isMinimumHeight(true),
 	m_rowHeight(0),
 	m_isDontEndAParagraphStyleForThisHardReturn(false)
-
 {
 	_read(input);
 }
@@ -72,10 +71,15 @@ WP6EOLGroup::~WP6EOLGroup()
 void WP6EOLGroup::_readContents(WPXInputStream *input)
 {
 	WPD_DEBUG_MSG(("WordPerfect: EOL Group: Reading Embedded Sub-Function Data\n"));
-	uint16_t sizeDeletableSubFunctionData;
 	unsigned int startPosition = input->tell();
-	sizeDeletableSubFunctionData = readU16(input);		
+	uint16_t sizeDeletableSubFunctionData = readU16(input);		
 	WPD_DEBUG_MSG(("WordPerfect: EOL Group: Size of Deletable Sub-Function Data: %ld,  Size of Deletable and Non-deletable sub-function data: %ld\n", (long) sizeDeletableSubFunctionData, (long) getSizeNonDeletable()));
+	if ((long)sizeDeletableSubFunctionData > (long)getSizeNonDeletable())
+	{
+		WPD_DEBUG_MSG(("WordPerfect: EOL Group: Possible corruption detected, bailing out\n"));
+		throw FileException();
+	}
+
 	input->seek(sizeDeletableSubFunctionData, WPX_SEEK_CUR);
 	while (input->tell() < (startPosition + getSizeNonDeletable()))
 	{
@@ -231,12 +235,16 @@ void WP6EOLGroup::_readContents(WPXInputStream *input)
 				numBytesToSkip = WP6_EOL_GROUP_DONT_END_A_PARAGRAPH_STYLE_FOR_THIS_HARD_RETURN_SIZE;
 				m_isDontEndAParagraphStyleForThisHardReturn = true;
 				break;
-			default:
-				// unsupported: shouldn't happen! an error may follow
-				// HACK: one document seems to have a completely bogus value within 
-				// pre-emptively throw a parsing exception
+			case 0x8e:
+			case 0x8f:
 				WPD_DEBUG_MSG(("WordPerfect: EOL Group Embedded Sub-Function: UNKNOWN SUBFUNCTION (%x) (BAD BAD BAD)\n", byte));
-				throw ParseException();
+				numBytesToSkip = readU16(input); // It seems that these two unknow sub-functions have their
+				                                 // length information embedded: <subfunction>[length]...[length]<subfunction>
+				break;
+			default:
+				WPD_DEBUG_MSG(("WordPerfect: EOL Group Embedded Sub-Function: UNKNOWN SUBFUNCTION (%x) (BAD BAD BAD)\n", byte));
+				WPD_DEBUG_MSG(("WordPerfect: Possible corruption detected, bailing out\n"));
+				throw FileException();
 		}			
 		
 		input->seek((startPosition2 + numBytesToSkip - 1 - input->tell()), WPX_SEEK_CUR);
