@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -33,7 +33,7 @@ class WP6ParagraphGroup_LineSpacingSubGroup : public WP6VariableLengthGroup_SubG
 {
 public:
 	WP6ParagraphGroup_LineSpacingSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	float m_lineSpacing;
@@ -43,7 +43,7 @@ class WP6ParagraphGroup_JustificationModeSubGroup : public WP6VariableLengthGrou
 {
 public:
 	WP6ParagraphGroup_JustificationModeSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	uint8_t m_justification;
@@ -53,7 +53,7 @@ class WP6ParagraphGroup_SpacingAfterParagraphSubGroup : public WP6VariableLength
 {
 public:
 	WP6ParagraphGroup_SpacingAfterParagraphSubGroup(WPXInputStream *input, const uint16_t sizeNonDeletable);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	float m_spacingAfterParagraphAbsolute;
@@ -66,7 +66,7 @@ class WP6ParagraphGroup_TabSetSubGroup : public WP6VariableLengthGroup_SubGroup
 public:
 	WP6ParagraphGroup_TabSetSubGroup(WPXInputStream *input);
 	~WP6ParagraphGroup_TabSetSubGroup();
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	bool m_isRelative;
@@ -79,7 +79,7 @@ class WP6ParagraphGroup_IndentFirstLineSubGroup : public WP6VariableLengthGroup_
 {
 public:
 	WP6ParagraphGroup_IndentFirstLineSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	int16_t m_firstLineOffset;
@@ -89,7 +89,7 @@ class WP6ParagraphGroup_LeftMarginAdjustmentSubGroup : public WP6VariableLengthG
 {
 public:
 	WP6ParagraphGroup_LeftMarginAdjustmentSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	int16_t m_leftMargin;
@@ -99,7 +99,7 @@ class WP6ParagraphGroup_RightMarginAdjustmentSubGroup : public WP6VariableLength
 {
 public:
 	WP6ParagraphGroup_RightMarginAdjustmentSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	int16_t m_rightMargin;
@@ -109,7 +109,7 @@ class WP6ParagraphGroup_OutlineDefineSubGroup : public WP6VariableLengthGroup_Su
 {
 public:
 	WP6ParagraphGroup_OutlineDefineSubGroup(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
+	void parse(WP6Listener *listener, const uint8_t numPrefixIDs, uint16_t const *prefixIDs) const;
 
 private:
 	uint16_t m_outlineHash;
@@ -123,8 +123,8 @@ class WP6ParagraphGroup : public WP6VariableLengthGroup
  public:
 	WP6ParagraphGroup(WPXInputStream *input);
 	~WP6ParagraphGroup();
-	virtual void _readContents(WPXInputStream *input);
-	virtual void parse(WP6HLListener *listener);
+	void _readContents(WPXInputStream *input);
+	void parse(WP6Listener *listener);
 private:
 	WP6VariableLengthGroup_SubGroup *m_subGroupData;
 };
