@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -23,9 +23,8 @@
  * Corel Corporation or Corel Corporation Limited."
  */
  
-#include "WP6LLListener.h"
-#include "WP6HLStylesListener.h"
-#include "WP6HLContentListener.h"
+#include "WP6StylesListener.h"
+#include "WP6ContentListener.h"
 #include "WP6Parser.h"
 #include "WPXHeader.h"
 #include "WP6Header.h"
@@ -56,12 +55,23 @@ WP6PrefixData * WP6Parser::getPrefixData(WPXInputStream *input)
 	}
 	catch(FileException)
 	{
+		WPD_DEBUG_MSG(("WordPerfect: Prefix Data most likely corrupted.\n"));
+		// TODO: Try to check packet after packet so that we try to recover at least the begining if the corruption is not at
+		//       the begining.
 		DELETEP(prefixData);
 		throw FileException();
 	}
-}
+	catch(...)
+	{
+		WPD_DEBUG_MSG(("WordPerfect: Prefix Data most likely corrupted. Trying to ignore.\n"));
+		// TODO: Try to check packet after packet so that we try to recover at least the begining if the corruption is not at
+		//       the begining.
+		DELETEP(prefixData);
+		return NULL;
+	}
+		}
 
-void WP6Parser::parse(WPXInputStream *input, WP6HLListener *listener)
+void WP6Parser::parse(WPXInputStream *input, WP6Listener *listener)
 {
 	listener->startDocument();
 	
@@ -111,7 +121,7 @@ static const uint16_t extendedInternationalCharacterMap[] =
 };
 
 // parseDocument: parses a document body (may call itself recursively, on other streams, or itself)
-void WP6Parser::parseDocument(WPXInputStream *input, WP6HLListener *listener)
+void WP6Parser::parseDocument(WPXInputStream *input, WP6Listener *listener)
 {
 	while (!input->atEOS())
 	{
@@ -143,8 +153,11 @@ void WP6Parser::parseDocument(WPXInputStream *input, WP6HLListener *listener)
 	}
 }
 
-void WP6Parser::parsePacket(WP6PrefixData *prefixData, int type, WP6HLListener *listener)
+void WP6Parser::parsePacket(WP6PrefixData *prefixData, int type, WP6Listener *listener)
 {
+	if (!prefixData)
+		return;
+
 	std::pair< MPDP_CIter, MPDP_CIter > * typeIterPair;
 	typeIterPair = prefixData->getPrefixDataPacketsOfType(type); 
 	if (typeIterPair->first != typeIterPair->second) 
@@ -155,8 +168,11 @@ void WP6Parser::parsePacket(WP6PrefixData *prefixData, int type, WP6HLListener *
 	DELETEP(typeIterPair);
 }
 
-void WP6Parser::parsePackets(WP6PrefixData *prefixData, int type, WP6HLListener *listener)
+void WP6Parser::parsePackets(WP6PrefixData *prefixData, int type, WP6Listener *listener)
 {
+	if (!prefixData)
+		return;
+
 	std::pair< MPDP_CIter, MPDP_CIter > * typeIterPair;
 
 	typeIterPair = prefixData->getPrefixDataPacketsOfType(type);
@@ -176,7 +192,7 @@ void WP6Parser::parsePackets(WP6PrefixData *prefixData, int type, WP6HLListener
 void WP6Parser::parse(WPXHLListenerImpl *listenerImpl)
 {	
 	WP6PrefixData * prefixData = NULL;
-	std::vector<WPXPageSpan *> pageList;
+	std::list<WPXPageSpan> pageList;
 	WPXTableList tableList;	
 
 	WPXInputStream *input = getInput();
@@ -187,13 +203,29 @@ void WP6Parser::parse(WPXHLListenerImpl *listenerImpl)
 		
 		// do a "first-pass" parse of the document
 		// gather table border information, page properties (per-page)
-		WP6HLStylesListener stylesListener(&pageList, tableList);
+		WP6StylesListener stylesListener(pageList, tableList);
 		stylesListener.setPrefixData(prefixData);
 		parse(input, &stylesListener);
+		
+		// postprocess the pageList == remove duplicate page spans due to the page breaks
+		std::list<WPXPageSpan>::iterator previousPage = pageList.begin();
+		for (std::list<WPXPageSpan>::iterator Iter=pageList.begin(); Iter != pageList.end(); /* Iter++ */)
+		{
+			if ((Iter != previousPage) && ((*previousPage)==(*Iter)))
+			{
+				(*previousPage).setPageSpan((*previousPage).getPageSpan() + (*Iter).getPageSpan());
+				Iter = pageList.erase(Iter);
+			}
+			else
+			{
+				previousPage = Iter;
+				Iter++;
+			}
+		}
 
 		// second pass: here is where we actually send the messages to the target app
 		// that are necessary to emit the body of the target document
-		WP6HLContentListener listener(&pageList, tableList, listenerImpl);
+		WP6ContentListener listener(pageList, tableList, listenerImpl);
 		listener.setPrefixData(prefixData);
 
 		// get the relevant initial prefix packets out of storage and tell them to parse
@@ -202,14 +234,10 @@ void WP6Parser::parse(WPXHLListenerImpl *listenerImpl)
 		parsePacket(prefixData, WP6_INDEX_HEADER_INITIAL_FONT, &listener);
 		parsePackets(prefixData, WP6_INDEX_HEADER_OUTLINE_STYLE, &listener);
 
-		parse(input, static_cast<WP6HLListener *>(&listener));
+		parse(input, static_cast<WP6Listener *>(&listener));
 
 		// cleanup section: free the used resources
 		delete prefixData;
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
-		{
-			delete *iterSpan;
-		}
 	}
 	catch(FileException)
 	{
@@ -217,11 +245,6 @@ void WP6Parser::parse(WPXHLListenerImpl *listenerImpl)
 
 		delete prefixData;
 		
-		for (std::vector<WPXPageSpan *>::iterator iterSpan = pageList.begin(); iterSpan != pageList.end(); iterSpan++)
-		{
-			delete *iterSpan;
-		}
-
 		throw FileException();
 	}
 }
