@@ -1,6 +1,7 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,7 +37,7 @@ class WP6Part
  public:
 	virtual ~WP6Part() {}
 	
-	static WP6Part * constructPart(WPXInputStream *input, uint8_t readVal);
+	static WP6Part * constructPart(WPXInputStream *input, const uint8_t readVal);
 	virtual void parse(WP6Listener *listener) = 0;
 };
 
