@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,10 +32,10 @@ class WP6UnsupportedFixedLengthGroup : public WP6FixedLengthGroup
 {
  public:
 	WP6UnsupportedFixedLengthGroup(WPXInputStream *input, uint8_t groupID);	
-	virtual void parse(WP6Listener *listener) {}
+	void parse(WP6Listener *listener) {}
 	
  protected:
-	virtual void _readContents(WPXInputStream *input) {}
+	void _readContents(WPXInputStream *input) {}
 };
 
 #endif /* WP6UNSUPPORTEDFIXEDLENGTHGROUP_H */
