@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003-2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003-2004 Marc Maurer (uwog@uwog.net)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -28,8 +28,10 @@
 #include "WPDocument.h"
 #include "WPXHeader.h"
 #include "WPXParser.h"
+#include "WP1Parser.h"
 #include "WP3Parser.h"
 #include "WP42Parser.h"
+#include "WP1Heuristics.h"
 #include "WP42Heuristics.h"
 #include "WP5Parser.h"
 #include "WP6Parser.h"
@@ -97,23 +99,47 @@ WPDConfidence WPDocument::isFileFormatSupported(WPXInputStream *input, bool part
 		header = WPXHeader::constructHeader(document);
 		if (header)
 		{
-			switch (header->getMajorVersion())
+			switch (header->getFileType())
 			{
-				case 0x00: // WP5
-				case 0x02: // WP6
-				case 0x03: // WP Mac 3.0-3.5
-				case 0x04: // WP Mac 3.5e
-					confidence = WPD_CONFIDENCE_EXCELLENT;
+				case 0x0a: // WordPerfect File
+					switch (header->getMajorVersion())
+					{
+						case 0x00: // WP5
+						case 0x02: // WP6+
+							confidence = WPD_CONFIDENCE_EXCELLENT;
+							break;
+						default:
+							// unhandled file format
+							confidence = WPD_CONFIDENCE_NONE;
+							break;
+					}
+					break;
+				case 0x2c: // WP Mac File
+					switch (header->getMajorVersion())
+					{
+						case 0x02: // WP Mac 2.x
+						case 0x03: // WP Mac 3.0-3.5
+						case 0x04: // WP Mac 3.5e
+							confidence = WPD_CONFIDENCE_EXCELLENT;
+							break;
+						default:
+							// unhandled file format
+							confidence = WPD_CONFIDENCE_NONE;
+							break;
+					}
 					break;
 				default:
-					// unhandled file format
+					// unhandled file type
 					confidence = WPD_CONFIDENCE_NONE;
 					break;
-			}
+			}						
 			DELETEP(header);
 		}
 		else
-			confidence = WP42Heuristics::isWP42FileFormat(input, partialContent);
+			confidence = WP1Heuristics::isWP1FileFormat(input, partialContent);
+			if (confidence != WPD_CONFIDENCE_EXCELLENT)
+				confidence = LIBWPD_MAX(confidence, WP42Heuristics::isWP42FileFormat(input, partialContent));
+			
 
 		// dispose of the reference to the ole input stream, if we allocated one
 		if (document != NULL && isDocumentOLE)
@@ -150,7 +176,7 @@ Parses the input stream content. It will make callbacks to the functions provide
 WPXHLListenerImpl class implementation when needed. This is often commonly called the
 'main parsing routine'.
 \param input The input stream
-\param listenerImpl A WPXHLListener implementation
+\param listenerImpl A WPXListener implementation
 */
 WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerImpl)
 {
@@ -182,27 +208,46 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 
 		if (header)
 		{
-			switch (header->getMajorVersion())
+			switch (header->getFileType())
 			{
-				case 0x00: // WP5
-					WPD_DEBUG_MSG(("WordPerfect: Using the WP5 parser.\n"));
-					parser = new WP5Parser(document, header);
-					parser->parse(listenerImpl);
+				case 0x0a: // WordPerfect File
+					switch (header->getMajorVersion())
+					{
+						case 0x00: // WP5
+							WPD_DEBUG_MSG(("WordPerfect: Using the WP5 parser.\n"));
+							parser = new WP5Parser(document, header);
+							parser->parse(listenerImpl);
+							break;
+						case 0x02: // WP6
+							WPD_DEBUG_MSG(("WordPerfect: Using the WP6 parser.\n"));
+							parser = new WP6Parser(document, header);
+							parser->parse(listenerImpl);
+							break;
+						default:
+							// unhandled file format
+							WPD_DEBUG_MSG(("WordPerfect: Unsupported file format.\n"));
+							break;
+					}
 					break;
-				case 0x02: // WP6
-					WPD_DEBUG_MSG(("WordPerfect: Using the WP6 parser.\n"));
-					parser = new WP6Parser(document, header);
-					parser->parse(listenerImpl);
-					break;
-				case 0x03: // WP Mac 3.0-3.5
-				case 0x04: // WP Mac 3.5e
-					WPD_DEBUG_MSG(("WordPerfect: Using the WP3 parser.\n"));
-					parser = new WP3Parser(document, header);
-					parser->parse(listenerImpl);
+				case 0x2c: // WP Mac File
+					switch (header->getMajorVersion())
+					{
+						case 0x02: // WP Mac 2.x
+						case 0x03: // WP Mac 3.0-3.5
+						case 0x04: // WP Mac 3.5e
+							WPD_DEBUG_MSG(("WordPerfect: Using the WP3 parser.\n"));
+							parser = new WP3Parser(document, header);
+							parser->parse(listenerImpl);
+							break;
+						default:
+							// unhandled file format
+							WPD_DEBUG_MSG(("WordPerfect: Unsupported file format.\n"));
+							break;
+					}
 					break;
 				default:
 					// unhandled file format
-					WPD_DEBUG_MSG(("WordPerfect: Unsupported file format.\n"));
+					WPD_DEBUG_MSG(("WordPerfect: Unsupported file type.\n"));
 					break;
 			}
 
@@ -213,10 +258,16 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 			// WP file formats prior to version 5.x do not contain a generic
 			// header which can be used to determine which parser to instanciate.
 			// Use heuristics to determine with some certainty if we are dealing with
-			// a file in the WP4.2 format.
-			int confidence = WP42Heuristics::isWP42FileFormat(document, false /* FIXME: allow for partial content */);
-
-			if (confidence == WPD_CONFIDENCE_GOOD || confidence == WPD_CONFIDENCE_EXCELLENT)
+			// a file in the WP4.2 format or WP Mac 1.x format.
+			if (WP1Heuristics::isWP1FileFormat(document, false) != WPD_CONFIDENCE_NONE)
+			{
+				WPD_DEBUG_MSG(("WordPerfect: Mostly likely the file format is WP Mac 1.x.\n\n"));
+				WPD_DEBUG_MSG(("WordPerfect: Using the WP Mac 1.x parser.\n\n"));
+				WP1Parser *parser = new WP1Parser(document);
+				parser->parse(listenerImpl);
+				DELETEP(parser);
+			}
+			else if (WP42Heuristics::isWP42FileFormat(document, false) != WPD_CONFIDENCE_NONE)
 			{
 				WPD_DEBUG_MSG(("WordPerfect: Mostly likely the file format is WP4.2.\n\n"));
 				WPD_DEBUG_MSG(("WordPerfect: Using the WP4.2 parser.\n\n"));
@@ -225,6 +276,7 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 				DELETEP(parser);
 			}
 			else
+
 				error = WPD_FILE_ACCESS_ERROR;
 		}
 	}
@@ -256,71 +308,3 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 
 	return error;
 }
-
-/*
-
-TOTALLY BROKEN, DO NOT USE!!!
-I MEAN IT!
-
-void WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerImpl, WPXFileType fileType)
-{
-	WPXParser *parser = NULL;
-	WPXHeader *header = NULL;
-
-	try
-	{
-		switch (fileType)
-		{
-			case WP42_DOCUMENT:
-				WPD_DEBUG_MSG(("WordPerfect: Using the WP42 parser.\n"));
-				parser = new WP42Parser(input);
-				parser->parse(listenerImpl);
-				DELETEP(parser);
-				break;
-			case WP5_DOCUMENT:
-				WPD_DEBUG_MSG(("WordPerfect: Using the WP5 parser.\n"));
-				header = new WPXHeader(input);
-				parser = new WP5Parser(input, header);
-				parser->parse(listenerImpl);
-				DELETEP(parser);
-				DELETEP(header);
-				break;
-			case WP6_DOCUMENT:
-				WPD_DEBUG_MSG(("WordPerfect: Using the WP6 parser.\n"));
-				header = new WPXHeader(input);
-				parser = new WP6Parser(input, header);
-				parser->parse(listenerImpl);
-				DELETEP(parser);
-				DELETEP(header);
-				break;
-			default:
-				// unhandled file format
-				WPD_DEBUG_MSG(("WordPerfect: Unsupported file format.\n"));
-				break;
-		}
-	}
-	catch (NoFileHeaderException)
-	{
-		// should not happen
-		DELETEP(parser);
-		DELETEP(header);
-		throw NoFileHeaderException();
-	}
-	catch (FileException)
-	{
-		DELETEP(parser);
-		DELETEP(header);
-		throw FileException();
-	}
-}
-
-*/
-
-
-/*
-TOTALLY BROKEN - MIGHT BE NICE TO HAVE
-
-WPXFileType WPDocument::WPXParser::getFileType(WPXInputStream *input)
-{
-
-}*/
