@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -24,51 +24,31 @@
  */
 
 #include <math.h>
+#include <algorithm>
 #include "WPXPageSpan.h"
 #include "libwpd_internal.h"
 
-const float WP6_DEFAULT_PAGE_MARGIN_TOP = 1.0f;
-const float WP6_DEFAULT_PAGE_MARGIN_BOTTOM = 1.0f;
-const float PAGE_MAX_DELTA = 0.05f;
+const float WPX_DEFAULT_PAGE_MARGIN_TOP = 1.0f;
+const float WPX_DEFAULT_PAGE_MARGIN_BOTTOM = 1.0f;
 const uint8_t DUMMY_INTERNAL_HEADER_FOOTER = 16;
 
 // precondition: 0 <= headerFooterType <= 3 (i.e.: we don't handle watermarks here)
-const WPXHeaderFooterType _convertHeaderFooterType(const uint8_t headerFooterType)
-{       
-	WPXHeaderFooterType wpxType;
-
-	(headerFooterType <= WP6_HEADER_FOOTER_GROUP_HEADER_B) ? wpxType = HEADER : wpxType = FOOTER;
-
-	return wpxType;
-}
-
-const WPXHeaderFooterOccurence _convertHeaderFooterOccurence(const uint8_t occurenceBits)
-{
-	if (occurenceBits & WP6_HEADER_FOOTER_GROUP_EVEN_BIT && occurenceBits & WP6_HEADER_FOOTER_GROUP_ODD_BIT)
-		return ALL;
-	else if (occurenceBits & WP6_HEADER_FOOTER_GROUP_EVEN_BIT)
-		return EVEN;
-	else
-		return ODD;
-
-}
-
 WPXHeaderFooter::WPXHeaderFooter(const WPXHeaderFooterType headerFooterType, const WPXHeaderFooterOccurence occurence, 
-				 const uint8_t internalType, const uint16_t textPID, WPXTableList tableList) :
+				 const uint8_t internalType, const WPXSubDocument * subDocument, WPXTableList tableList) :
 	m_type(headerFooterType),
 	m_occurence(occurence),
 	m_internalType(internalType),
-	m_textPID(textPID),
+	m_subDocument(subDocument),
 	m_tableList(tableList)
 {
 }
 
 WPXHeaderFooter::WPXHeaderFooter(const WPXHeaderFooterType headerFooterType, const WPXHeaderFooterOccurence occurence, 
-				 const uint8_t internalType, const uint16_t textPID) :
+				 const uint8_t internalType, const WPXSubDocument * subDocument) :
 	m_type(headerFooterType),
 	m_occurence(occurence),
 	m_internalType(internalType),
-	m_textPID(textPID)
+	m_subDocument(subDocument)
 {
 }
 
@@ -76,13 +56,14 @@ WPXHeaderFooter::WPXHeaderFooter(const WPXHeaderFooter &headerFooter) :
 	m_type(headerFooter.getType()),
 	m_occurence(headerFooter.getOccurence()),
 	m_internalType(headerFooter.getInternalType()),
-	m_textPID(headerFooter.getTextPID()),
+	m_subDocument(headerFooter.getSubDocument()),
 	m_tableList(headerFooter.getTableList())
 {
 }
 
 WPXHeaderFooter::~WPXHeaderFooter()
 {
+//	delete m_subDocument;
 }
 
 WPXPageSpan::WPXPageSpan() :
@@ -91,17 +72,32 @@ WPXPageSpan::WPXPageSpan() :
 	m_formOrientation(PORTRAIT),
 	m_marginLeft(1.0f),
 	m_marginRight(1.0f),
-	m_marginTop(WP6_DEFAULT_PAGE_MARGIN_TOP),
-	m_marginBottom(WP6_DEFAULT_PAGE_MARGIN_BOTTOM),
+	m_marginTop(WPX_DEFAULT_PAGE_MARGIN_TOP),
+	m_marginBottom(WPX_DEFAULT_PAGE_MARGIN_BOTTOM),
 	m_pageSpan(1)
 {
-	for (int i=0; i<WP6_NUM_HEADER_FOOTER_TYPES; i++)
+	for (int i=0; i<WPX_NUM_HEADER_FOOTER_TYPES; i++)
 		m_isHeaderFooterSuppressed[i]=false;
 }
 
+WPXPageSpan::WPXPageSpan(const WPXPageSpan &page) :
+	m_formLength(page.getFormLength()),
+	m_formWidth(page.getFormWidth()),
+	m_formOrientation(page.getFormOrientation()),
+	m_marginLeft(page.getMarginLeft()),
+	m_marginRight(page.getMarginRight()),
+	m_marginTop(page.getMarginTop()),
+	m_marginBottom(page.getMarginBottom()),
+	m_headerFooterList(page.getHeaderFooterList()),
+	m_pageSpan(page.getPageSpan())
+{
+	for (int i=0; i<WPX_NUM_HEADER_FOOTER_TYPES; i++)
+		m_isHeaderFooterSuppressed[i] = page.getHeaderFooterSuppression(i);	
+}
+
 // NB: this is not a literal "clone" function: it is contingent on the side margins that are passed,
 // and suppression variables are not copied
-WPXPageSpan::WPXPageSpan(WPXPageSpan &page, float paragraphMarginLeft, float paragraphMarginRight) :
+WPXPageSpan::WPXPageSpan(const WPXPageSpan &page, float paragraphMarginLeft, float paragraphMarginRight) :
 	m_formLength(page.getFormLength()),
 	m_formWidth(page.getFormWidth()),
 	m_formOrientation(page.getFormOrientation()),
@@ -109,10 +105,10 @@ WPXPageSpan::WPXPageSpan(WPXPageSpan &page, float paragraphMarginLeft, float par
 	m_marginRight(page.getMarginRight()+paragraphMarginRight),
 	m_marginTop(page.getMarginTop()),
 	m_marginBottom(page.getMarginBottom()),
-	m_pageSpan(page.getPageSpan()),
-	m_headerFooterList(page.getHeaderFooterList())
+	m_headerFooterList(page.getHeaderFooterList()),
+	m_pageSpan(page.getPageSpan())
 {
-	for (int i=0; i<WP6_NUM_HEADER_FOOTER_TYPES; i++)
+	for (int i=0; i<WPX_NUM_HEADER_FOOTER_TYPES; i++)
 		m_isHeaderFooterSuppressed[i] = false;	
 }
 
@@ -121,40 +117,41 @@ WPXPageSpan::~WPXPageSpan()
 }
 
 
-void WPXPageSpan::setHeaderFooter(const uint8_t headerFooterType, const uint8_t occurenceBits, 
-				  const uint16_t textPID, WPXTableList tableList)
+void WPXPageSpan::setHeaderFooter(const WPXHeaderFooterType type, const uint8_t headerFooterType, const WPXHeaderFooterOccurence occurence, 
+				  const  WPXSubDocument * subDocument, WPXTableList tableList)
 {
-        WPXHeaderFooterType wpxType = _convertHeaderFooterType(headerFooterType);
-	WPXHeaderFooterOccurence wpxOccurence = _convertHeaderFooterOccurence(occurenceBits);
-	WPXHeaderFooter headerFooter(wpxType, wpxOccurence, headerFooterType, textPID, tableList);
-	switch (wpxOccurence) 
+	WPXHeaderFooter headerFooter(type, occurence, headerFooterType, subDocument, tableList);
+	switch (occurence) 
 	{
 	case ALL:
-		_removeHeaderFooter(wpxType, ODD); _removeHeaderFooter(wpxType, EVEN); _removeHeaderFooter(wpxType, ALL);
+	case NEVER:
+		_removeHeaderFooter(type, ODD); _removeHeaderFooter(type, EVEN); _removeHeaderFooter(type, ALL);
 		break;
 	case ODD:
-		_removeHeaderFooter(wpxType, ODD); _removeHeaderFooter(wpxType, ALL);
+		_removeHeaderFooter(type, ODD); _removeHeaderFooter(type, ALL);
 		break;
 	case EVEN:
-		_removeHeaderFooter(wpxType, EVEN); _removeHeaderFooter(wpxType, ALL);
+		_removeHeaderFooter(type, EVEN); _removeHeaderFooter(type, ALL);
 		break;		
 	}
-	m_headerFooterList.push_back(headerFooter);
 
-	bool containsHFLeft = _containsHeaderFooter(wpxType, ODD);
-	bool containsHFRight = _containsHeaderFooter(wpxType, EVEN);
+	if ((occurence != NEVER) && (subDocument))
+		m_headerFooterList.push_back(headerFooter);
+
+	bool containsHFLeft = _containsHeaderFooter(type, ODD);
+	bool containsHFRight = _containsHeaderFooter(type, EVEN);
 
 	WPD_DEBUG_MSG(("Contains HFL: %i HFR: %i\n", containsHFLeft, containsHFRight));
 	if (containsHFLeft && !containsHFRight)
 	{
 		WPD_DEBUG_MSG(("Inserting dummy header right\n"));
-		WPXHeaderFooter dummyHeader(wpxType, EVEN, DUMMY_INTERNAL_HEADER_FOOTER, 0);
+		WPXHeaderFooter dummyHeader(type, EVEN, DUMMY_INTERNAL_HEADER_FOOTER, 0);
 		m_headerFooterList.push_back(dummyHeader);
 	}
 	else if (!containsHFLeft && containsHFRight)
 	{
 		WPD_DEBUG_MSG(("Inserting dummy header left\n"));
-		WPXHeaderFooter dummyHeader(wpxType, ODD, DUMMY_INTERNAL_HEADER_FOOTER, 0);
+		WPXHeaderFooter dummyHeader(type, ODD, DUMMY_INTERNAL_HEADER_FOOTER, 0);
 		m_headerFooterList.push_back(dummyHeader);
 	}
 }
@@ -167,6 +164,7 @@ void WPXPageSpan::makeConsistent(int startingPageNumber)
 {
 	if (!(startingPageNumber % 2))
 	{
+	// not sure whether this has any use (Fridrich) ?
 	}
 }
 
@@ -178,16 +176,6 @@ void WPXPageSpan::_removeHeaderFooter(WPXHeaderFooterType type, WPXHeaderFooterO
 			WPD_DEBUG_MSG(("WordPerfect: Removing header/footer element of type: %i since it is identical to %i\n",(*iter).getType(), type));
 			m_headerFooterList.erase(iter);
 			return;
-#if 0
-			WPD_DEBUG_MSG(("WordPerfect: Now our list looks like this:\n"));
-			for (std::vector<WPXHeaderFooter>::const_iterator iter1 = m_headerFooterList.begin(); 
-			     iter1 != m_headerFooterList.end(); 
-			     iter1++) 
-			{
-			WPD_DEBUG_MSG(("WordPerfect: El.: %i %i\n", (*iter1).getType(), (*iter1).getTextPID()));
-			}
-			return;
-#endif
 		}
 	}
 }
@@ -203,42 +191,50 @@ bool WPXPageSpan::_containsHeaderFooter(WPXHeaderFooterType type, WPXHeaderFoote
 	return false;
 }
 
+inline bool operator==(const WPXHeaderFooter &headerFooter1, const WPXHeaderFooter &headerFooter2)
+{
+	return ((headerFooter1.getType() == headerFooter2.getType()) && 
+		(headerFooter1.getSubDocument() == headerFooter2.getSubDocument()) &&
+		(headerFooter1.getOccurence() == headerFooter2.getOccurence()) &&
+		(headerFooter1.getInternalType() == headerFooter2.getInternalType()) );
+}
+
 bool operator==(const WPXPageSpan &page1, const WPXPageSpan &page2)
 {
-	if (fabs(page1.getMarginLeft() - page2.getMarginLeft()) > PAGE_MAX_DELTA ||
-	    fabs(page1.getMarginRight() - page2.getMarginRight())  > PAGE_MAX_DELTA ||
-	    fabs(page1.getMarginTop() - page2.getMarginTop()) > PAGE_MAX_DELTA ||
-	    fabs(page1.getMarginBottom() - page2.getMarginBottom()) > PAGE_MAX_DELTA)
+	if ((page1.getMarginLeft() != page2.getMarginLeft()) || (page1.getMarginRight() != page2.getMarginRight()) ||
+	    (page1.getMarginTop() != page2.getMarginTop())|| (page1.getMarginBottom() != page2.getMarginBottom()))
 		return false;
 
 
-	for (int i=0; i<WP6_NUM_HEADER_FOOTER_TYPES; i++) {
+	for (int i=0; i<WPX_NUM_HEADER_FOOTER_TYPES; i++) {
 		if (page1.getHeaderFooterSuppression(i) != page2.getHeaderFooterSuppression(i))
 			return false;
 	}
 
 	// NOTE: yes this is O(n^2): so what? n=4 at most
-	const std::vector<WPXHeaderFooter> headerFooterList1 = page1.getHeaderFooterList();	
-	for (std::vector<WPXHeaderFooter>::const_iterator iter1 = headerFooterList1.begin(); 
-	     iter1 != headerFooterList1.end(); 
-	     iter1++) 
+	const std::vector<WPXHeaderFooter> headerFooterList1 = page1.getHeaderFooterList();
+	const std::vector<WPXHeaderFooter> headerFooterList2 = page2.getHeaderFooterList();
+	std::vector<WPXHeaderFooter>::const_iterator iter1;		
+	std::vector<WPXHeaderFooter>::const_iterator iter2;		
+
+	for (iter1 = headerFooterList1.begin(); iter1 != headerFooterList1.end(); iter1++)
 	{
-		const std::vector<WPXHeaderFooter> headerFooterList2 = page2.getHeaderFooterList();	
-		std::vector<WPXHeaderFooter>::const_iterator iter2;
-		for (iter2 = headerFooterList2.begin(); iter2 != headerFooterList2.end(); iter2++) 
-		{
-			WPD_DEBUG_MSG(("WordPerfect: WPXPageSpan ==  header/footer comparison (%i, %i))\n", 
-					       (*iter1).getTextPID(), (*iter2).getTextPID()));
-			if ((*iter1).getType() == (*iter2).getType() && (*iter1).getTextPID() == (*iter2).getTextPID())
-			{
-				WPD_DEBUG_MSG(("WordPerfect: WPXPageSpan == found same header/footer (%i, %i), breaking)\n", 
-					       (*iter1).getTextPID(), (*iter2).getTextPID()));
-				break;
-			}
-		}		
-		if (iter2 == headerFooterList2.end())
+		if (std::find(headerFooterList2.begin(), headerFooterList2.end(), (*iter1)) == headerFooterList2.end())
 			return false;
 	}
+	
+	// If we came here, we know that every header/footer that is found in the first page span is in the second too.
+	// But this is not enought for us to know whether the page spans are equal. Now we have to check in addition
+	// whether every header/footer that is in the second one is in the first too. If someone wants to optimize this,
+	// (s)he is most welcome :-)
+	
+	for (iter2 = headerFooterList2.begin(); iter2 != headerFooterList2.end(); iter2++)
+	{
+		if (std::find(headerFooterList1.begin(), headerFooterList1.end(), (*iter2)) == headerFooterList1.end())
+			return false;
+	}
+
+	
 	WPD_DEBUG_MSG(("WordPerfect: WPXPageSpan == comparison finished, found no differences\n"));
 
 	return true;
