@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2004 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2004 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2005 Net Integration Technologies (http://www.net-itech.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -168,19 +168,15 @@ WPXBoolProperty::WPXBoolProperty(const bool val)  :
 
 WPXString WPXBoolProperty::getStr() const 
 { 
-	WPXString str; 
-
 	if (getInt())
-		str.sprintf("true"); 
+		return "true"; 
 	else
-		str.sprintf("false"); 
-
-	return str; 
+		return "false"; 
 }
 
 WPXProperty * WPXBoolProperty::clone() const
 {
-	return new WPXBoolProperty((const bool)getInt());
+	return new WPXBoolProperty((const bool)(getInt() != 0));
 }
 
 WPXFloatProperty::WPXFloatProperty(const float val) :
