@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002, 2005 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002, 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002, 2005 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002, 2004 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -24,13 +24,14 @@
  */
 #include "libwpd_internal.h"
 #include "WPXStream.h"
+#include <ctype.h>
 
 uint8_t readU8(WPXInputStream *input)
 {
 	size_t numBytesRead;
 	uint8_t const * p = input->read(sizeof(uint8_t), numBytesRead);
 	
-  	if (numBytesRead != sizeof(uint8_t))
+  	if (!p || numBytesRead != sizeof(uint8_t))
   		throw FileException();
 
 	return WPD_LE_GET_GUINT8(p);
@@ -41,7 +42,7 @@ int8_t read8(WPXInputStream *input)
 	size_t numBytesRead;
 	int8_t const * p = (int8_t const *) input->read(sizeof(int8_t), numBytesRead);
 
-  	if (numBytesRead != sizeof(int8_t))
+  	if (!p || numBytesRead != sizeof(int8_t))
   		throw FileException();
 
 	return (int8_t)*(p);
@@ -52,7 +53,7 @@ uint16_t readU16(WPXInputStream *input, bool bigendian)
 	size_t numBytesRead;
 	uint16_t const *val = (uint16_t const *)input->read(sizeof(uint16_t), numBytesRead);
 
-	if (numBytesRead != sizeof(uint16_t))
+	if (!val || numBytesRead != sizeof(uint16_t))
   		throw FileException();
 
 	if (bigendian)
@@ -65,7 +66,7 @@ uint32_t readU32(WPXInputStream *input, bool bigendian)
 	size_t numBytesRead;
 	uint32_t const *val = (uint32_t const *)input->read(sizeof(uint32_t), numBytesRead);
 
-	if (numBytesRead != sizeof(uint32_t))
+	if (!val || numBytesRead != sizeof(uint32_t))
   		throw FileException();
 
 	if (bigendian)
@@ -73,6 +74,24 @@ uint32_t readU32(WPXInputStream *input, bool bigendian)
 	return WPD_LE_GET_GUINT32(val);
 }
 
+WPXString readPascalString(WPXInputStream *input)
+{
+	int pascalStringLength = readU8(input);
+	WPXString tmpString;
+	for (int i=0; i<pascalStringLength; i++)
+		tmpString.append((char)readU8(input));
+	return tmpString;
+}
+
+WPXString readCString(WPXInputStream *input)
+{
+	WPXString tmpString;
+	char character;
+	while ((character = (char)readU8(input)) != '\0')
+		tmpString.append(character);
+	return tmpString;
+}
+
 // the ascii map appears stupid, but we need the const 16-bit data for now
 static const uint16_t asciiMap[] =
 {
@@ -495,77 +514,77 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 		if (character < WP6_NUM_MULTINATIONAL_CHARACTERS)
 			*chars = &multinationalWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 		
 	case WP6_PHONETIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_PHONETIC_CHARACTERS)
 			*chars = &phoneticWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_BOX_DRAWING_CHARACTER_SET:
 		if (character < WP6_NUM_BOX_DRAWING_CHARACTERS)
 			*chars = &boxdrawingWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_TYPOGRAPHIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_TYPOGRAPHIC_CHARACTERS)
 			*chars = &typographicWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_ICONIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_ICONIC_CHARACTERS)
 			*chars = &iconicWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_MATH_SCIENTIFIC_CHARACTER_SET:
 		if (character < WP6_NUM_MATH_SCIENTIFIC_CHARACTERS)
 			*chars = &mathWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_MATH_SCIENTIFIC_EXTENDED_CHARACTER_SET:
 		if (character < WP6_NUM_MATH_SCIENTIFIC_EXTENDED_CHARACTERS)
 			*chars = &mathextWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_GREEK_CHARACTER_SET:
 		if (character < WP6_NUM_GREEK_CHARACTERS)
 			*chars = &greekWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_HEBREW_CHARACTER_SET:
 		if (character < WP6_NUM_HEBREW_CHARACTERS)
 			*chars = &hebrewWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_CYRILLIC_CHARACTER_SET:
 		if (character < WP6_NUM_CYRILLIC_CHARACTERS)
 			*chars = &cyrillicWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_JAPANESE_CHARACTER_SET:
 		if (character < WP6_NUM_JAPANESE_CHARACTERS)
 			*chars = &japaneseWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_TIBETAN_CHARACTER_SET:
@@ -576,7 +595,7 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 			return i;
 		}
 		else {
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 			return 1;
 		}
 
@@ -584,19 +603,19 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 		if (character < WP6_NUM_ARABIC_CHARACTERS)
 			*chars = &arabicWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP6_ARABIC_SCRIPT_CHARACTER_SET:
 		if (character < WP6_NUM_ARABIC_SCRIPT_CHARACTERS)
 			*chars = &arabicScriptWP6[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 	}
 
 	// last resort: return whitespace
-	*chars = &asciiMap[' '];
+	*chars = &asciiMap[32];
 	return 1;
 }
 
@@ -838,63 +857,63 @@ int extendedCharacterWP5ToUCS2(uint8_t character,
 		if (character < WP5_NUM_INTERNATIONAL_1_CHARACTERS)
 			*chars = &international1WP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_INTERNATIONAL_2_CHARACTER_SET:
 		if (character < WP5_NUM_INTERNATIONAL_2_CHARACTERS)
 			*chars = &international2WP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_BOX_DRAWING_CHARACTER_SET:
 		if (character < WP5_NUM_BOX_DRAWING_CHARACTERS)
 			*chars = &boxdrawingWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_TYPOGRAPHIC_SYMBOL_CHARACTER_SET:
 		if (character < WP5_NUM_TYPOGRAPHIC_CHARACTERS)
 			*chars = &typographicWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_ICONIC_SYMBOL_CHARACTER_SET:
 		if (character < WP5_NUM_ICONIC_CHARACTERS)
 			*chars = &iconicWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_MATH_SCIENTIFIC_CHARACTER_SET:
 		if (character < WP5_NUM_MATH_SCIENTIFIC_CHARACTERS)
 			*chars = &mathWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_MATH_SCIENTIFIC_EXTENDED_CHARACTER_SET:
 		if (character < WP5_NUM_MATH_SCIENTIFIC_EXTENDED_CHARACTERS)
 			*chars = &mathextWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_GREEK_CHARACTER_SET:
 		if (character < WP5_NUM_GREEK_CHARACTERS)
 			*chars = &greekWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_HEBREW_CHARACTER_SET:
 		if (character < WP5_NUM_HEBREW_CHARACTERS)
 			*chars = &hebrewWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 
 	case WP5_CYRILLIC_CHARACTER_SET:
@@ -906,15 +925,32 @@ int extendedCharacterWP5ToUCS2(uint8_t character,
 		if (character < WP5_NUM_JAPANESE_CHARACTERS)
 			*chars = &japaneseWP5[character];
 		else
-			*chars = &asciiMap[' '];
+			*chars = &asciiMap[32];
 		return 1;
 	}
 
 	// last resort: return whitespace
-	*chars = &asciiMap[' '];
+	*chars = &asciiMap[32];
 	return 1;
 }
 
+#include "WPXFileStructure.h"
+#include "libwpd_math.h"
+
+uint16_t fixedPointToWPUs(const uint32_t fixedPointNumber)
+{
+	int16_t fixedPointNumberIntegerPart = (int16_t)((fixedPointNumber & 0xFFFF0000) >> 16);
+	float fixedPointNumberFractionalPart = (float)((double)(fixedPointNumber & 0x0000FFFF)/(double)0xFFFF);
+	uint16_t numberWPU = (uint16_t)rint((((float)fixedPointNumberIntegerPart + fixedPointNumberFractionalPart)*50)/3);
+	return numberWPU;
+}
+
+float fixedPointToFloat(const uint32_t fixedPointNumber)
+{
+	int16_t fixedPointNumberIntegerPart = (int16_t)((fixedPointNumber & 0xFFFF0000) >> 16);
+	float fixedPointNumberFractionalPart = (float)((double)(fixedPointNumber & 0x0000FFFF)/(double)0xFFFF);
+	return ((float)fixedPointNumberIntegerPart + fixedPointNumberFractionalPart);
+}	
 
 _RGBSColor::_RGBSColor(uint8_t r, uint8_t g, uint8_t b, uint8_t s)
 	:	m_r(r),
@@ -932,6 +968,30 @@ _RGBSColor::_RGBSColor()
 {
 }
 
+_RGBSColor::_RGBSColor(uint16_t red, uint16_t green, uint16_t blue)
+{
+	int minRGB = red;
+	if (minRGB > green)
+		minRGB = green;
+	if (minRGB > blue)
+		minRGB = blue;
+		
+	if (minRGB >= 65535)
+	{
+		m_r = 255;
+		m_g = 255;
+		m_b = 255;
+		m_s = 100;
+	}
+	else
+	{
+		m_r = (uint8_t)rint(255*((double)(red - minRGB))/((double)(65535 - minRGB)));
+		m_g = (uint8_t)rint(255*((double)(green - minRGB))/((double)(65535 - minRGB)));
+		m_b = (uint8_t)rint(255*((double)(blue - minRGB))/((double)(65535 - minRGB)));
+		m_s = (uint8_t)rint(100*((double)(65535 - minRGB))/(double)65535);
+	}		
+}
+
 _WPXTabStop::_WPXTabStop(float position, WPXTabAlignment alignment, uint16_t leaderCharacter, uint8_t leaderNumSpaces)
 	:	m_position(position),
 		m_alignment(alignment),
@@ -954,3 +1014,158 @@ _WPXColumnDefinition::_WPXColumnDefinition()
 		m_rightGutter(0.0f)
 {
 }
+
+_WPXColumnProperties::_WPXColumnProperties()
+	:	m_attributes(0x00000000),
+		m_alignment(0x00)
+{
+}
+
+// HACK: this function is really cheesey
+int _extractNumericValueFromRoman(const char romanChar)
+{
+	switch (romanChar)
+	{
+	case 'I':
+	case 'i':
+		return 1;
+	case 'V':
+	case 'v':
+		return 5;
+	case 'X':
+	case 'x':
+		return 10;
+	default:
+		throw ParseException();
+	}
+	return 1;
+}
+
+// _extractDisplayReferenceNumberFromBuf: given a nuWP6_DEFAULT_FONT_SIZEmber string in UCS2 represented
+// as letters, numbers, or roman numerals.. return an integer value representing its number
+// HACK: this function is really cheesey
+// NOTE: if the input is not valid, the output is unspecified
+int _extractDisplayReferenceNumberFromBuf(const WPXString &buf, const WPXNumberingType listType)
+{
+	if (listType == LOWERCASE_ROMAN || listType == UPPERCASE_ROMAN)
+	{
+		int currentSum = 0;
+		int lastMark = 0;
+		WPXString::Iter i(buf);
+		for (i.rewind(); i.next();)
+		{
+			int currentMark = _extractNumericValueFromRoman(*(i()));
+			if (lastMark < currentMark) {
+				currentSum = currentMark - lastMark;
+			}
+			else
+				currentSum+=currentMark;
+			lastMark = currentMark;
+		}
+		return currentSum;
+	}
+	else if (listType == LOWERCASE || listType == UPPERCASE)
+	{
+		// FIXME: what happens to a lettered list that goes past z? ah
+		// the sweet mysteries of life
+		if (buf.len()==0)
+			throw ParseException();
+		char c = buf.cstr()[0];
+		if (listType==LOWERCASE)
+			c = toupper(c);
+		return (c - 64);
+	}
+	else if (listType == ARABIC)
+	{
+		int currentSum = 0;
+		WPXString::Iter i(buf);
+		for (i.rewind(); i.next();)
+		{
+			currentSum *= 10;
+			currentSum+=(*(i())-48);
+		}
+		return currentSum;
+	}
+
+	return 1;
+}
+
+WPXNumberingType _extractWPXNumberingTypeFromBuf(const WPXString &buf, const WPXNumberingType putativeWPXNumberingType)
+{
+	WPXString::Iter i(buf);
+	for (i.rewind(); i.next();)
+	{
+		if ((*(i()) == 'I' || *(i()) == 'V' || *(i()) == 'X') &&
+		    (putativeWPXNumberingType == LOWERCASE_ROMAN || putativeWPXNumberingType == UPPERCASE_ROMAN))
+			return UPPERCASE_ROMAN;
+		else if ((*(i()) == 'i' || *(i()) == 'v' || *(i()) == 'x') &&
+		    (putativeWPXNumberingType == LOWERCASE_ROMAN || putativeWPXNumberingType == UPPERCASE_ROMAN))
+			return LOWERCASE_ROMAN;
+		else if (*(i()) >= 'A' && *(i()) <= 'Z')
+			return UPPERCASE;
+		else if (*(i()) >= 'a' && *(i()) <= 'z')
+			return LOWERCASE;
+	}
+
+	return ARABIC;
+}
+
+WPXString _numberingTypeToString(WPXNumberingType t)
+{
+	WPXString sListTypeSymbol("1");
+	switch (t)
+	{
+	case ARABIC:
+		sListTypeSymbol = "1";
+		break;	
+	case LOWERCASE:
+		sListTypeSymbol = "a";
+		break;	
+	case UPPERCASE:
+		sListTypeSymbol = "A";
+		break;	
+ 	case LOWERCASE_ROMAN:
+		sListTypeSymbol = "i";
+		break;	
+ 	case UPPERCASE_ROMAN:
+		sListTypeSymbol = "I";
+		break;
+	}
+
+	return sListTypeSymbol;
+}
+
+/* Mapping of Apple's MacRoman character set in Unicode (UCS2) 
+ * used in the WordPerfect Macintosh file format */
+ 
+const uint16_t macintoshCharacterMap[] =
+{
+  0x0020, 0x0021, 0x0022, 0x0023, 0x0024, 0x0025, 0x0026, 0x0027,
+  0x0028, 0x0029, 0x002a, 0x002b, 0x002c, 0x002d, 0x002e, 0x002f,
+  0x0030, 0x0031, 0x0032, 0x0033, 0x0034, 0x0035, 0x0036, 0x0037,
+  0x0038, 0x0039, 0x003a, 0x003b, 0x003c, 0x003d, 0x003e, 0x003f,
+  0x0040, 0x0041, 0x0042, 0x0043, 0x0044, 0x0045, 0x0046, 0x0047,
+  0x0048, 0x0049, 0x004a, 0x004b, 0x004c, 0x004d, 0x004e, 0x004f,
+  0x0050, 0x0051, 0x0052, 0x0053, 0x0054, 0x0055, 0x0056, 0x0057,
+  0x0058, 0x0059, 0x005a, 0x005b, 0x005c, 0x005d, 0x005e, 0x005f,
+  0x0060, 0x0061, 0x0062, 0x0063, 0x0064, 0x0065, 0x0066, 0x0067,
+  0x0068, 0x0069, 0x006a, 0x006b, 0x006c, 0x006d, 0x006e, 0x006f,
+  0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077,
+  0x0078, 0x0079, 0x007a, 0x007b, 0x007c, 0x007d, 0x007e, 0x007f,
+  0x00c4, 0x00c5, 0x00c7, 0x00c9, 0x00d1, 0x00d6, 0x00dc, 0x00e1,
+  0x00e0, 0x00e2, 0x00e4, 0x00e3, 0x00e5, 0x00e7, 0x00e9, 0x00e8,
+  0x00ea, 0x00eb, 0x00ed, 0x00ec, 0x00ee, 0x00ef, 0x00f1, 0x00f3,
+  0x00f2, 0x00f4, 0x00f6, 0x00f5, 0x00fa, 0x00f9, 0x00fb, 0x00fc,
+  0x2020, 0x00b0, 0x00a2, 0x00a3, 0x00a7, 0x2022, 0x00b6, 0x00df,
+  0x00ae, 0x00a9, 0x2122, 0x00b4, 0x00a8, 0x2260, 0x00c6, 0x00d8,
+  0x221e, 0x00b1, 0x2264, 0x2265, 0x00a5, 0x00b5, 0x2202, 0x2211,
+  0x220f, 0x03c0, 0x222b, 0x00aa, 0x00ba, 0x03a9, 0x00e6, 0x00f8,
+  0x00bf, 0x00a1, 0x00ac, 0x221a, 0x0192, 0x2248, 0x2206, 0x00ab,
+  0x00bb, 0x2026, 0x00a0, 0x00c0, 0x00c3, 0x00d5, 0x0152, 0x0153,
+  0x2013, 0x2014, 0x201c, 0x201d, 0x2018, 0x2019, 0x00f7, 0x25ca,
+  0x00ff, 0x0178, 0x2044, 0x20ac, 0x2039, 0x203a, 0xfb01, 0xfb02,
+  0x2021, 0x00b7, 0x201a, 0x201e, 0x2030, 0x00c2, 0x00ca, 0x00c1,
+  0x00cb, 0x00c8, 0x00cd, 0x00ce, 0x00cf, 0x00cc, 0x00d3, 0x00d4,
+  0xf8ff, 0x00d2, 0x00da, 0x00db, 0x00d9, 0x0131, 0x02c6, 0x02dc,
+  0x00af, 0x02d8, 0x02d9, 0x02da, 0x00b8, 0x02dd, 0x02db, 0x02c7
+};
